--
-- PostgreSQL database dump
--

-- Started on 2010-05-05 17:51:41

SET statement_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;

--
-- TOC entry 7 (class 2615 OID 32234)
-- Name: rutility; Type: SCHEMA; Schema: -; Owner: postgres
--

CREATE SCHEMA rutility;


ALTER SCHEMA rutility OWNER TO postgres;

SET search_path = rutility, pg_catalog;

--
-- TOC entry 883 (class 1255 OID 68210)
-- Dependencies: 7 1170 1196 1285
-- Name: upgis_ptval(public.raster, integer, public.geometry); Type: FUNCTION; Schema: rutility; Owner: postgres
--

CREATE FUNCTION upgis_ptval(param_rast public.raster, param_bnum integer, param_pt public.geometry) RETURNS double precision
    LANGUAGE plpgsql STABLE
    AS $$
  DECLARE 
    var_result float = 0; var_rid integer;
    var_cols integer; var_rows integer;var_c integer;  var_r integer;
    var_upx float; var_upy float; var_sizex float; var_sizey float; 
  BEGIN
  	SELECT ST_Width(param_rast), ST_Height(param_rast), ST_UpperLeftX(param_rast), ST_UpperLeftY(param_rast), ST_PixelSizeX(param_rast), ST_PixelSizeY(param_rast)
      INTO var_cols, var_rows, var_upx, var_upy, var_sizex, var_sizey
    WHERE ST_Intersects(ST_Envelope(param_rast), param_pt) 
    LIMIT 1;
    
    IF var_cols IS NULL THEN -- doesn't intersect
      var_result := NULL;
    ELSE
    SELECT 1 + floor( ( (ST_X(param_pt) - var_upx) / (var_sizex*var_cols) )*var_cols )::integer, 1 + floor( ( (ST_Y(param_pt) - var_upy)/(var_sizey*var_rows) )*var_rows )::integer
      INTO var_c, var_r;
      IF var_c = 0 THEN var_c := 1; END IF;
      IF var_r = 0 THEN var_r := 1; END IF; 
      IF var_c = var_cols + 1 THEN var_c := var_cols; END IF;
      IF var_r = var_rows + 1 THEN var_r := var_rows; END IF;
    
      SELECT ST_Value(param_rast,param_bnum, var_c , var_r )
        INTO var_result;
    END IF;
    --RAISE NOTICE 'col: %, row: %, val: %, pt: %', var_c, var_r, var_result, ST_AsText(param_pt);

    RETURN var_result;
  END
 $$;


ALTER FUNCTION rutility.upgis_ptval(param_rast public.raster, param_bnum integer, param_pt public.geometry) OWNER TO postgres;

--
-- TOC entry 2884 (class 0 OID 0)
-- Dependencies: 883
-- Name: FUNCTION upgis_ptval(param_rast public.raster, param_bnum integer, param_pt public.geometry); Type: COMMENT; Schema: rutility; Owner: postgres
--

COMMENT ON FUNCTION upgis_ptval(param_rast public.raster, param_bnum integer, param_pt public.geometry) IS 'This function takes as input raster, band_num, point geometry and returns the band pixel value of the cell the point falls in.  It assumes the spatial reference system of the geometry and raster are the same.';


--
-- TOC entry 805 (class 1255 OID 32235)
-- Dependencies: 1285 1170 1196 7
-- Name: upgis_rastertopolygon(public.raster, integer[], integer); Type: FUNCTION; Schema: rutility; Owner: postgres
--

CREATE FUNCTION upgis_rastertopolygon(param_rast public.raster, param_bands integer[], param_sampling integer) RETURNS public.geometry
    LANGUAGE plpgsql IMMUTABLE
    AS $$
DECLARE 
 var_rows integer := ST_Height(param_rast);
 var_cols integer := ST_Width(param_rast);
 var_pixsizex float := ST_PixelSizeX(param_rast);
 var_pixsizey float := ST_PixelSizeY(param_rast);
 var_pixpoly geometry := ST_MakeEnvelope(ST_UpperLeftX(param_rast),ST_UpperLeftY(param_rast), 
		ST_UpperLeftX(param_rast) + var_pixsizex*param_sampling, ST_UpperLeftY(param_rast) + var_pixsizey*param_sampling, ST_SRID(param_rast));
 var_result geometry;
 var_nodatavalue float[]  := ARRAY(SELECT 
		ST_BandNoDataValue(param_rast, param_bands[i])
			FROM generate_series(1, array_upper(param_bands,1)) As i  );
BEGIN
	SELECT ST_Union(ST_Translate(var_pixpoly, x*var_pixsizex, y*var_pixsizey))
		INTO var_result
			FROM generate_series(1,var_cols,param_sampling) As x
					CROSS JOIN generate_series(1,var_rows,param_sampling) As y
		WHERE EXISTS(SELECT 1 FROM generate_series(1, array_upper(param_bands,1)) As i
				WHERE ST_Value(param_rast, param_bands[i],x,y) <> var_nodatavalue[i]);
	RETURN var_result;
END
$$;


ALTER FUNCTION rutility.upgis_rastertopolygon(param_rast public.raster, param_bands integer[], param_sampling integer) OWNER TO postgres;

--
-- TOC entry 876 (class 1255 OID 43597)
-- Dependencies: 7 1196 1170 1285
-- Name: upgis_rastertopolygon(public.raster, integer[], integer, double precision[]); Type: FUNCTION; Schema: rutility; Owner: postgres
--

CREATE FUNCTION upgis_rastertopolygon(param_rast public.raster, param_bands integer[], param_sampling integer, param_valrange double precision[]) RETURNS public.geometry
    LANGUAGE plpgsql IMMUTABLE
    AS $$
DECLARE 
 var_rows integer := ST_Height(param_rast);
 var_cols integer := ST_Width(param_rast);
 var_pixsizex float := ST_PixelSizeX(param_rast);
 var_pixsizey float := ST_PixelSizeY(param_rast);
 var_pixpoly geometry := ST_MakeEnvelope(ST_UpperLeftX(param_rast),ST_UpperLeftY(param_rast), 
		ST_UpperLeftX(param_rast) + var_pixsizex*param_sampling, ST_UpperLeftY(param_rast) + var_pixsizey*param_sampling, ST_SRID(param_rast));
 var_result geometry;
 
BEGIN
	SELECT ST_Union(ST_Translate(var_pixpoly, x*var_pixsizex, y*var_pixsizey))
		INTO var_result
			FROM generate_series(1,var_cols,param_sampling) As x
					CROSS JOIN generate_series(1,var_rows,param_sampling) As y
		WHERE EXISTS(SELECT 1 FROM generate_series(1, array_upper(param_bands,1)) As i
				WHERE ST_Value(param_rast, param_bands[i],x,y) BETWEEN param_valrange[i][1] AND param_valrange[i][2] );
	RETURN var_result;
END
$$;


ALTER FUNCTION rutility.upgis_rastertopolygon(param_rast public.raster, param_bands integer[], param_sampling integer, param_valrange double precision[]) OWNER TO postgres;

-- Completed on 2010-05-05 17:51:41

--
-- PostgreSQL database dump complete
--

