CREATE TABLE ch13.travels(travelid integer PRIMARY KEY, title text, pt_start geography(POINT(4326)), pt_end geography(POINT,4326), path geography(LINESTRING,4326), max_dev_m integer, move_inc integer, max_duration interval); 

INSERT INTO ch13.travels(travelid, title, pt_start, pt_end, max_dev_m, move_inc, max_duration) VALUES(1, 'San CA to Reno NV', ST_GeogFromText('POINT(-122.68 37.75)'), ST_GeogFromText('POINT(-119.78 39.5)'), 1000, 1000, interval '1 minute');


CREATE OR REPLACE FUNCTION ch13.travel_pixval(param_pos geography(POINT,4326))
  RETURNS float AS
  $$
  DECLARE 
    var_result float = 0; 
  BEGIN
  SELECT rutility.upgis_ptval(k.rast, 1, geometry(param_pos)) As val
  	INTO var_result
    FROM ch13.usdem AS k
    WHERE ST_Intersects(geography(ST_Envelope(k.rast)), param_pos) 
   LIMIT 1;
  RETURN var_result;
END
$$
  LANGUAGE 'plpgsql' STABLE;

  
-- Plan the travel and for each move update the record and move on
-- SELECT ch13.travel_planner(1);
CREATE OR REPLACE FUNCTION ch13.travel_planner(param_travelid integer)
	RETURNS text AS
$$
	DECLARE 
		var_starttime timestamp with time zone := clock_timestamp();
		var_result text := ' Start ' ||  var_starttime;
		var_curpos geography;
		var_tarpos geography;
		var_tv ch13.travels;
		var_loops integer := 0;
		var_geog_us geography;
		var_corridor geometry;
		var_circle geometry;
		var_curpv float;
	
  BEGIN
    SELECT * INTO var_tv FROM ch13.travels WHERE travel_id = param_travel_id;
    IF var_tv.path IS NOT NULL THEN
      var_curpos := geography(ST_EndPoint(geometry(var_tv.path)));
    ELSE
      var_curpos := var_tv.pt_start;
    END IF;
    var_curpv := ch13.travel_pixval(var_curpos);
    -- 1 keep path within buffer of straight  
var_corridor := ST_Buffer(geography(ST_MakeLine(geometry(var_tv.pt_start), geometry(var_tv.pt_end))), var_tv.max_dev_m); 
    -- 2 loop til 10 m of destination or time out 
    WHILE (var_starttime + var_tv.max_duration) > clock_timestamp() AND ST_Distance(var_curpos,var_tv.pt_end) > 1000   LOOP
      -- 3 test points around a circle
      var_circle := geometry(ST_Buffer(var_curpos, var_tv.move_inc)) ;
      WITH pos_runs AS
        (SELECT geography(foo.geom) As pot_loc FROM ST_DumpPoints(var_circle) As foo WHERE foo.path[2] < ST_NPoints(var_circle) AND ST_Intersects(foo.geom, var_corridor) AND (var_tv.path IS NULL OR NOT ST_Crosses(ST_MakeLine(geometry(var_curpos),foo.geom), geometry(var_tv.path)))),
        
        pos_runs_pd AS
        (SELECT pot_loc, ch13.travel_pixval(pot_loc) As pixval, ST_Distance(pot_loc, var_tv.pt_end) As dist_end
            FROM pos_runs
        WHERE ST_Distance(pot_loc, var_tv.pt_end) < ST_Distance(var_curpos, var_tv.pt_end)  )
     -- 4 find best new point       
  SELECT pot_loc
        INTO var_tarpos
        FROM pos_runs_pd
        -- 5 penalize more for going up than going down
        ORDER BY CASE WHEN pixval > var_curpv THEN (pixval - var_curpv)*100 ELSE (var_curpv - pixval)*20 END + dist_end*0.1 LIMIT 1;
       
      IF var_tv.path IS NULL THEN
        var_tv.path := geography(ST_MakeLine(geometry(var_curpos), geometry(var_tarpos))) ;
      ELSEIF var_tarpos IS NOT NULL THEN
        var_tv.path := geography(ST_AddPoint(geometry(var_tv.path), geometry(var_tarpos)) ) ;
      ELSE
        EXIT;
      END IF;
      var_curpos := var_tarpos; var_loops := var_loops + 1;
      var_curpv := ch13.travel_pixval(var_curpos); 
    END LOOP;
    -- 6 update path with accumulated
    UPDATE ch13.travels SET path = var_tv.path WHERE ch13.travels.travel_id = var_tv.travel_id; 
    var_result := var_result || ' To End: ' || clock_timestamp() || ' in loops: ' || var_loops || ' Duration %' || age(clock_timestamp() ,var_starttime) ; 
  RETURN var_result;
END
$$
LANGUAGE 'plpgsql' VOLATILE;







  

  
